import re
from abc import abstractmethod
from typing import Dict, List, Optional, Union

from ..type_specs import Hotkeys, HotkeysWithModes
from .sources import SourceBase


class Extractor:
    has_modes: Optional[bool] = None
    sources: Optional[Dict[str, List[SourceBase]]] = None
    required: Optional[List[SourceBase]] = None

    def __init__(self):
        if self.required is not None:
            if not any([source.check() for source in self.required]):
                raise OSError(f"No checks of {self.required} succeeded.")

    def fetch(self) -> Dict[str, List[str]]:
        if self.sources is None:
            raise TypeError("Can't fetch, 'sources' attribute is None.")

        out = {}
        for k, sources in self.sources.items():
            out[k] = [source.fetch() for source in sources if source.check()]
        return out

    @abstractmethod
    def extract(
        self, fetched: Dict[str, List[str]]
    ) -> Union[Hotkeys, HotkeysWithModes]:
        """Must return a dict with structure:
        if has_modes: {'mode': {'hotkey': 'action'}}
        else: {'hotkey': 'action'}
        """
        pass


class SectionExtract:
    """Helper methods to assist in handling man page source documents."""

    def find_sections(self, content: str, pattern: str = r"\.SH"):
        sections = self.find_between(content, pattern)
        return dict([self.split_title(s) for s in sections])

    @staticmethod
    def find_between(content: str, pattern: str):
        return re.findall(rf"(?<={pattern})(.*?)(?={pattern}|$)", content, re.DOTALL)

    @staticmethod
    def split_title(content: str, pattern="\n"):
        content = content.lstrip()
        content = content.replace('"', "")
        first_split = content.index(pattern)
        title = content[:first_split]
        content = content[first_split:]
        return title.strip(), content.lstrip()



import gzip
import importlib
import shlex
import subprocess
from abc import abstractmethod
from pathlib import Path
from shutil import which
from typing import Union


# Content sources
class SourceBase:
    @abstractmethod
    def fetch(self) -> str:
        pass

    @abstractmethod
    def check(self) -> bool:
        pass


class PythonModule(SourceBase):
    def __init__(self, module_name: str):
        self.module_name = module_name
        self._module = None

    def check(self) -> bool:
        try:
            importlib.import_module(self.module_name)
        except ModuleNotFoundError:
            return False
        return True

    def __repr__(self) -> str:
        return repr(self.module_name)


class Command(SourceBase):
    def __init__(self, command: str):
        self.source = shlex.split(command)

    def fetch(self) -> str:
        return subprocess.check_output(self.source).decode("utf8")

    def check(self) -> bool:
        return which(self.source[0]) is not None

    def __repr__(self) -> str:
        return repr(" ".join(self.source))


class File(SourceBase):
    def __init__(self, file_path: Union[Path, str]):
        if isinstance(file_path, str):
            file_path = Path(file_path)
        self.source = file_path

    def fetch(self) -> str:
        with open(self.source, "r") as fp:
            return fp.read()

    def check(self) -> bool:
        return self.source.is_file()

    def __repr__(self) -> str:
        return repr(str(self.source))


class Manpage(SourceBase):
    def __init__(self, page: str):
        self.source = page

    def fetch(self) -> str:
        man_page_path = subprocess.check_output(["man", "-w", self.source]).decode(
            "utf8"
        )
        man_page_path = Path(man_page_path.rstrip())
        with gzip.open(man_page_path, "rb") as gfp:
            return gfp.read().decode("utf8")

    def check(self) -> bool:
        all_pages = (
            subprocess.check_output(["man", "-k", "."]).decode("utf8").split("\n")
        )
        all_pages = [page.split()[0] for page in all_pages if page]
        return self.source in all_pages

    def __repr__(self) -> str:
        return repr(self.source)



import os
import re
from pathlib import Path

from ..type_specs import Hotkeys
from .base import Extractor
from .sources import Command, File

# need to add custom config bindings


class Mpv(Extractor):
    required = [Command("mpv")]
    sources = {
        "system": [
            File("/usr/share/doc/mpv/input.conf"),
            File("/etc/mpv/input.conf"),
            File("/usr/local/etc/mpv/input.conf"),
        ],
        "user": [File(Path(os.environ["XDG_CONFIG_HOME"]) / "mpv" / "input.conf")],
    }
    has_modes = False

    @staticmethod
    def _clean_action(string):
        # replace whitespaces with space
        string = re.sub(r"\s+", " ", string)
        return string

    def extract(self, fetched) -> Hotkeys:
        out = {}
        content_key_action = re.compile(r"^#(\S+)\s+(.*)\n", re.MULTILINE)
        # remove this stray line to make the regex easier
        line_remove = re.compile(r"#default-bindings.*")
        # extract from all the fetched files
        for content in sum(fetched.values(), []):
            if content:
                content = re.sub(line_remove, "", content)
                for match in re.finditer(content_key_action, content):
                    out[match[1]] = self._clean_action(match[2])
        return out



import re

from ..type_specs import HotkeysWithModes
from .base import Extractor
from .sources import Command


class Vim(Extractor):
    required = [Command("vim")]
    # map won't show autocmd added hotkeys
    sources = {
        "user": [
            Command(f'vim -e +"redir >> /dev/stdout | map | redir END" -scq'),
        ]
    }
    has_modes = True

    def extract(self, fetched) -> HotkeysWithModes:
        # convert vim mode notation to human, from :help map
        mode_map = {
            " ": "Normal, Visual, Select, Operator-pending",
            "n": "Normal",
            "v": "Visual, Select",
            "s": "Select",
            "x": "Visual",
            "o": "Operator-pending",
            "!": "Insert, Command-line",
            "i": "Insert",
            "l": '":lmap" mappings for Insert, Command-line and Lang-Arg',
            "c": "Command-line",
            "t": "Terminal-Job",
        }
        content = fetched["user"][0]
        # get the key/action
        content_key_action = re.compile(r"(.)\s+(\S+)\s+[@\&\*]?\s*(.*?)\n")
        out = {}
        for match in re.finditer(content_key_action, content):
            key = match[2]
            # remove <Plug> keys
            if not key.startswith("<Plug>"):
                mode = mode_map[match[1]]
                action = match[3]
                if not mode in out.keys():
                    out[mode] = {}
                out[mode][key] = action
        return out



from abc import abstractmethod


class Display:
    def __init__(self, hk_dict, has_modes):
        self.hk_dict = hk_dict
        self.has_modes = has_modes

    @abstractmethod
    def show(self, mode=None):
        pass



import json

from .base import Display


class Text(Display):
    def show(self, modes=None):
        if self.has_modes and modes is not None:
            if not (isinstance(modes, list)):
                modes = [modes]
            hk_dict = {mode: self.hk_dict[mode] for mode in modes}
        else:
            hk_dict = self.hk_dict

        for k, v in hk_dict.items():
            if isinstance(v, dict):
                for key, action in v.items():
                    print(f"{k}: {key}: {action}")
            else:
                print(f"{k}: {v}")


class Json(Display):
    def show(self, modes=None):
        if self.has_modes and modes is not None:
            if not (isinstance(modes, list)):
                modes = [modes]
            hk_dict = {mode: self.hk_dict[mode] for mode in modes}
        else:
            hk_dict = self.hk_dict
        print(json.dumps(hk_dict))



import argparse
import sys

from . import __version__, displays, extractors


def main():
    parser = argparse.ArgumentParser(
        description="Hotkey aggregator. All your hotkeys in one place."
    )

    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-le",
        "--list-extractors",
        help="List available hotkey extractors.",
        action="store_true",
        dest="list_extractors",
    )
    group.add_argument(
        "-ld",
        "--list-displays",
        help="List available display methods.",
        action="store_true",
        dest="list_displays",
    )
    parser.add_argument(
        "extractor",
        nargs="?",
        help="Extract hotkeys using extractor.",
        choices=[i.lower() for i in extractors.__all__],
    )
    parser.add_argument(
        "-d",
        "--display",
        help="Display method.",
        default="Text",
        choices=[i.lower() for i in displays.__all__],
    )
    parser.add_argument(
        "-m",
        "--modes",
        help="Filter by mode, if supported by extractor.",
        action="append",
        default=None,
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        help="Show hag version and exit.",
        version=f"%(prog)s {__version__}",
    )
    args = parser.parse_args()

    if args.list_displays:
        for i in displays.__all__:
            print(i.lower())
    elif args.list_extractors:
        for extractor in extractors.__all__:
            try:
                # if the install check passes
                getattr(extractors, extractor)()
                print(extractor.lower())
            except OSError:
                pass
    else:
        if args.extractor is None:
            parser.print_help()
            sys.exit(1)
        # get the desired extrator and display
        Extractor = getattr(extractors, args.extractor.title())
        Display = getattr(displays, args.display.title())

        # load hotkeys
        extractor = Extractor()
        fetched = extractor.fetch()
        hotkeys = extractor.extract(fetched)
        # display
        Display(hotkeys, has_modes=extractor.has_modes).show(modes=args.modes)


if __name__ == "__main__":
    main()



import re

from ..type_specs import HotkeysWithModes
from .base import Extractor
from .sources import Command


# TODO: this is broken
class Lazygit(Extractor):
    required = [Command("lazygit")]
    sources = {"user": [Command("lazygit -c")]}
    has_modes = True

    def extract(self, fetched) -> HotkeysWithModes:
        out = {}
        section_re = re.compile(r"(?<=keybinding:).*(?=os)", re.DOTALL)
        mode_re = re.compile(r"^  (\S*?):$", re.MULTILINE)
        key_action_re = re.compile(r"^    (\S*?): [\"']?(\S*?)[\"']?$", re.MULTILINE)
        # keep the keybinding section
        match = re.search(section_re, fetched["user"][0])
        if match is None:
            raise TypeError("Section match is None")
        content = match[0]
        for line in content.split("\n"):
            # get the mode
            mode_match = re.match(mode_re, line)
            if mode_match:
                mode = mode_match[1]
                out[mode] = {}
            else:
                # get the key/action
                key_action_match = re.match(key_action_re, line)
                if key_action_match:
                    out[mode][key_action_match[1]] = key_action_match[2]
        return out



import os
import re
from pathlib import Path

from ..type_specs import HotkeysWithModes
from .base import Extractor
from .sources import Command, File, Manpage


class Sxiv(Extractor):
    required = [Command("sxiv")]
    sources = {
        "default": [Manpage("sxiv")],
        "key_handler": [
            File(Path(os.environ["XDG_CONFIG_HOME"]) / "sxiv" / "exec" / "key-handler")
        ],
    }
    has_modes = True

    @staticmethod
    def _clean_action(string):
        return string.replace("\n", " ").strip()

    @staticmethod
    def _clean_key(string):
        return string.replace(r"\-", "-").strip()

    def extract(self, fetched) -> HotkeysWithModes:
        content = fetched["default"][0]
        # to select section from manpage
        content_section = re.compile(r"\.SH KEYBOARD COMMANDS.*?\.SH", re.DOTALL)
        # to split the section in different mode
        content_modes = re.compile(r"\.SS\s(.*?)\n(.*?)(?=(\.SS)|$)", re.DOTALL)
        # get each key/action from the mode section
        mode_key_action = re.compile(r"\.B[R]?\s(.*?\n)(.*?)(=?\.TP\n)", re.DOTALL)
        # clean up some stray strings
        content_clean = re.compile(r'(", ")|(\.I[R]? )')
        # only keep the desired man page section
        match = re.search(content_section, content)
        if match is None:
            raise TypeError("Section match is None")
        content = match[0]
        # clean
        content = re.sub(content_clean, "", content)
        out = {}
        # find all the modes
        for mode_match in re.finditer(content_modes, content):
            mode = mode_match[1]
            mode_c = mode_match[2]
            # add mode
            if mode not in out.keys():
                out[mode] = {}
            # find all the key/actions
            for key_action in re.finditer(mode_key_action, mode_c):
                key = self._clean_key(key_action[1])
                action = self._clean_action(key_action[2])
                out[mode][key] = action

        # extract key_handler
        if fetched["key_handler"]:
            out["key-handler"] = {}
            content = fetched["key_handler"][0]
            # remove comments
            comments = re.compile(r"\s*#.*")
            # get key/action
            content_cases = re.compile(r"\s\"(.*)\"\)\s*\n\s*(.*(?=;;))")
            # clean
            content = re.sub(comments, "", content)
            cases = re.finditer(content_cases, content)
            for case in cases:
                out["key-handler"][case[1]] = case[2]
        return out



import re

from ..type_specs import HotkeysWithModes
from .base import Extractor, SectionExtract
from .sources import Command, Manpage


class Zathura(SectionExtract, Extractor):
    required = [Command("zathura")]
    sources = {"default": [Manpage("zathura")]}
    has_modes = True

    @staticmethod
    def _clean_action(string):
        string = string.replace("\\fB", "").replace("\\fP", "")
        string = string.replace("\\-", "-")
        string = string[: string.index("\n")] + "."
        return string

    @staticmethod
    def _clean_key(string):
        string = string.replace("\\-", "-").replace("\\(aq", "'")
        return string

    def extract(self, fetched) -> HotkeysWithModes:
        content = fetched["default"][0]
        # select section from manpage
        content_section = re.compile(r"\.SH MOUSE AND KEY BINDINGS.*?\.SH", re.DOTALL)
        # split section in modes
        content_modes = re.compile(r"\.sp\n(.*?)\n(.*?)(?=\.sp|$)", re.DOTALL)
        # get the key/actions
        mode_key_action = re.compile(r"\.B(.*?)\n(.*?)\n")
        match = re.search(content_section, content)
        if match is None:
            raise TypeError("Section match is None")
        content = match[0]
        out = {}
        for mode_match in re.finditer(content_modes, content):
            mode = mode_match[1]
            mode_content = mode_match[2]
            if mode not in out.keys():
                out[mode] = {}
            for key_action in re.finditer(mode_key_action, mode_content):
                key = self._clean_key(key_action[1])
                out[key] = key_action[2]
        return out



import os
import re
from pathlib import Path

from ..type_specs import Hotkeys
from .base import Extractor
from .sources import Command, File


class Lf(Extractor):
    required = [Command("lf")]
    sources = {
        "default": [Command("lf -doc")],
        "user": [File(Path(os.environ["XDG_CONFIG_HOME"]) / "lf" / "lfrc")],
    }
    has_modes = False

    def extract(self, fetched) -> Hotkeys:
        content_clean = re.compile(r".*?((boolean)|(string)|(integer)).*\n")
        content_section = re.compile(r"Reference.*?Configuration", re.DOTALL)
        default_key_action = re.compile(r"\s*(.*?)\s*\(default (.*)\)")
        map_key_action = re.compile(r"\s*map\s(\w+)\s*(.*)\n")
        content_clean_quote = re.compile(r'(?<!")\'(?!")')
        # extract the default bindings
        # get the desired section
        match = re.search(content_section, fetched["default"][0])
        if match is None:
            raise TypeError("Section match is None.")
        content = match[0]
        # clean the problematic lines
        content = re.sub(content_clean, "", content)
        # remove the quotes could be improved
        content = re.sub(content_clean_quote, "", content)
        content = content.replace("'\"'", '"').replace('"\'"', "'")
        out = {}
        for match in re.finditer(default_key_action, content):
            out[match[2]] = match[1]

        for match in re.finditer(map_key_action, content):
            out[match[1]] = match[2]

        # extract the user bindings
        if fetched["user"]:
            content = fetched["user"][0]
            for match in re.finditer(map_key_action, content):
                out[match[1]] = match[2]
        return out



import os

from ..type_specs import HotkeysWithModes
from .base import Extractor
from .sources import PythonModule


class Qutebrowser(Extractor):
    required = [PythonModule("qutebrowser")]
    has_modes = True

    def fetch(self) -> HotkeysWithModes:
        # based on: https://github.com/qutebrowser/qutebrowser/blob/master/qutebrowser/config/configinit.py#L40
        from qutebrowser.config import config, configdata, configfiles
        from qutebrowser.utils import standarddir

        standarddir._init_dirs()
        configdata.init()
        yaml_config = configfiles.YamlConfig()
        config.instance = config.Config(yaml_config=yaml_config)
        config.key_instance = config.KeyConfig(config.instance)
        config.val = config.ConfigContainer(config.instance)
        yaml_config.setParent(config.instance)

        config_file = standarddir.config_py()
        if os.path.exists(config_file):
            configfiles.read_config_py(config_file)
        else:
            configfiles.read_autoconfig()

        configfiles.init()
        modes = config.val.bindings.default.keys()
        return {
            mode: {
                str(key): value
                for key, value in config.key_instance.get_bindings_for(mode).items()
            }
            for mode in modes
        }

    def extract(self, fetched: HotkeysWithModes) -> HotkeysWithModes:
        return fetched



from ..type_specs import HotkeysWithModes
from .base import Extractor
from .sources import PythonModule


class Vimiv(Extractor):
    required = [PythonModule("vimiv")]
    has_modes = True

    def fetch(self) -> HotkeysWithModes:
        from vimiv import api

        return {mode: dict(keys) for mode, keys in api.keybindings.items()}

    def extract(self, fetched) -> HotkeysWithModes:
        return fetched



import os
import re
from pathlib import Path

from ..type_specs import Hotkeys
from .base import Extractor
from .sources import Command, File


class Sxhkd(Extractor):
    required = [Command("sxhkd")]
    sources = {
        "user": [
            File(
                Path(
                    os.environ.get(
                        "XDG_CONFIG_HOME",
                        Path(os.environ["HOME"]) / ".config",
                    )
                )
                / "sxhkd"
                / "sxhkdrc"
            )
        ]
    }
    has_modes = False

    @staticmethod
    def _clean_fetched(content):
        # remove blank lines
        lines = [line for line in content.split("\n") if not re.match(r"\W?#", line)]
        lines = [line for line in lines if line != ""]
        return lines

    @staticmethod
    def _clean_action(string):
        string = string.strip()
        return string

    @staticmethod
    def _clean_key(string):
        string = string.replace(" + ", "+")
        return string

    def extract(self, fetched) -> Hotkeys:
        fetched = self._clean_fetched(fetched["user"][0])
        fetched = iter(fetched)
        out = {}
        for key in fetched:
            actions = []
            action = self._clean_action(next(fetched))
            actions.append(action)
            while action.endswith("\\"):
                action = self._clean_action(next(fetched))
                actions.append(action)
            out[self._clean_key(key)] = "".join(
                [ac.replace("//", "") for ac in actions]
            )
        return out



from ..type_specs import Hotkeys
from .base import Extractor
from .sources import Command


class Zsh(Extractor):
    required = [Command("zsh")]
    # TODO: this doesn't work for my setup, my config is not sourced :(
    sources = {"user": [Command("zsh -ci bindkey")]}
    has_modes = False

    @staticmethod
    def _clean_key(string: str):
        string = string.replace('"', "")
        return string

    def extract(self, fetched) -> Hotkeys:
        content = fetched["user"][0]
        out = {}
        for line in content.split("\n"):
            if line:
                key, action = line.split(" ")
                out[self._clean_key(key)] = action
        return out



import re

from ..type_specs import Hotkeys
from .base import Extractor
from .sources import Command


class Rofi(Extractor):
    required = [Command("rofi")]
    sources = {"user": [Command("rofi -dump-config")]}
    has_modes = False

    @staticmethod
    def _clean_key(string):
        string = string.lstrip()[1:-1]
        return string

    def extract(self, fetched) -> Hotkeys:
        content_key_action = re.compile(r".*?(((kb)|(ml)|(me))-.*?):\s+\"(.*?)\"")
        content = fetched["user"][0]
        out = {}
        for match in re.finditer(content_key_action, content):
            out[match[6]] = match[1]
        return out



from .sources import Command
from .vim import Vim


class Neovim(Vim):
    required = [Command("nvim")]
    sources = {
        "user": [
            Command(f'nvim -e +"redir>>/dev/stdout | map | redir END" -scq'),
        ]
    }
    has_modes = True



import re

from ..type_specs import HotkeysWithModes
from .base import Extractor
from .sources import Command, Manpage


class Termite(Extractor):
    required = [Command("termite")]
    sources = {"default": [Manpage("termite")]}
    has_modes = True

    def extract(self, fetched) -> HotkeysWithModes:
        content = fetched["default"][0]
        # remove some stray groff tags
        content_clean = re.compile(r"(\\f[PBI])")
        # get the desired section
        content_section = re.compile(r"\.SH KEYBINDINGS.*?\.SH", re.DOTALL)
        # split the section into modes
        content_modes = re.compile(r"\.SS\s+(.*?)\n(.*?)(?=\.SS|$)", re.DOTALL)
        # get the key/actions
        mode_key_action = re.compile(r'\.IP\s+"(.*?)"\n(.*?)\n')
        # clean
        content = re.sub(content_clean, "", content)
        # get section
        match = re.search(content_section, content)
        if match is None:
            raise TypeError("Section match is None")
        content = match[0]
        out = {}
        # find modes
        for mode_match in re.finditer(content_modes, content):
            mode = mode_match[1]
            mode_content = mode_match[2]
            if mode not in out.keys():
                out[mode] = {}
            # find keys
            for key_action in re.finditer(mode_key_action, mode_content):
                out[mode][key_action[1]] = key_action[2]
        return out



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hag', 'hag.displays', 'hag.extractors', 'hag.extractors.sources']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['hag = hag.__main__:main']}

setup_kwargs = {
    'name': 'hag',
    'version': '0.1.1',
    'description': 'A hotkey aggregator. All your hotkeys in one place.',
    'long_description': '<h1 align="center">hag</h1>\n<h5 align="center">A Hotkey AGgregator.</h5>\n<p align="center">\n  <a href="https://pypi.org/project/hag/"><img src="https://img.shields.io/pypi/v/hag"></a>\n  <a href="./LICENSE.md"><img src="https://img.shields.io/badge/license-MIT-blue.svg"></a>\n  <img src="https://img.shields.io/badge/platform-linux-informational">\n</p>\n\n`hag` is a a hotkey aggregator, it tries its best to extract the hotkeys of your software and display them.\n\nIt does so by parsing the config files, man pages, command outputs, ... depending on the software.\n\n## Programs\n\nBellow is a list of implemented software:\n\n- Lazygit\n- Lf\n- Mpv\n- Neovim\n- Qutebrowser\n- Rofi\n- Sxhkd\n- Sxiv\n- Termite\n- Vim\n- Vimiv\n- Zathura\n- Zsh\n\n## Installation\n\n`hag` is meant to be relatively minimal, as such it doesn\'t have any dependencies.\n\n```\npip install hag\n```\n\nIf you just want to use the CLI interface, consider using [`pipx`](https://github.com/pypa/pipx).\n\n```\npipx install hag\n```\n\n## Usage\n\n```\n$ hag --help\nusage: hag [-h] [-le | -ld] [-d {json,text}] [-m MODES] [-v] [{lazygit,lf,mpv,neovim,qutebrowser,rofi,sxhkd,sxiv,termite,vim,vimiv,zathura,zsh}]\n\nHotkey aggregator. All your hotkeys in one place.\n\npositional arguments:\n  {lazygit,lf,mpv,neovim,qutebrowser,rofi,sxhkd,sxiv,termite,vim,vimiv,zathura,zsh}\n                        Extract hotkeys using extractor.\n\noptions:\n  -h, --help            show this help message and exit\n  -le, --list-extractors\n                        List available hotkey extractors.\n  -ld, --list-displays  List available display methods.\n  -d {json,text}, --display {json,text}\n                        Display method.\n  -m MODES, --modes MODES\n                        Filter by mode, if supported by extractor.\n  -v, --version         Show hag version and exit.\n```\n\n### Examples\n\nA few example uses:\n\n- List [`sxhkd`](https://github.com/baskerville/sxhkd) hotkeys:\n  ```sh\n  hag sxhkd\n  ```\n- Display `sxhkd` hotkeys in json format and format with [`jq`](https://github.com/stedolan/jq):\n\n  ```sh\n  hag sxhkd -d json | jq\n  ```\n\n- Show `vim` Normal and Visual mode hotkeys in [`rofi`](https://github.com/davatorium/rofi):\n  ```sh\n  hag vim -m Normal | rofi -dmenu\n  ```\n- Use `rofi` to select software and show hotkeys:\n  ```sh\n  extractor="$(hag -le | rofi -dmenu)" && hag "$extractor" | rofi -dmenu\n  ```\n\n# Contributing\n\nIf you want to add support for your favourite software, feel free to open issues/PRs!\n',
    'author': 'Loic Coyle',
    'author_email': 'loic.coyle@hotmail.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/loiccoyle/hag',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



from .lazygit import Lazygit
from .lf import Lf
from .mpv import Mpv
from .neovim import Neovim
from .qutebrowser import Qutebrowser
from .rofi import Rofi
from .sxhkd import Sxhkd
from .sxiv import Sxiv
from .termite import Termite
from .vim import Vim
from .vimiv import Vimiv
from .zathura import Zathura
from .zsh import Zsh

__all__ = [
    "Lazygit",
    "Lf",
    "Mpv",
    "Neovim",
    "Qutebrowser",
    "Rofi",
    "Sxhkd",
    "Sxiv",
    "Termite",
    "Vim",
    "Vimiv",
    "Zathura",
    "Zsh",
]
